             .title        "Namcot 108/109/118/119 + 74xx32 Reader Plugin"


;05/14/07 
;Written by BootGod
;Version 1.0

;10/01/09
;Added check for single screen mirroring (INES 154)
;BootGod
             
             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

address:       .equ 00e0h
address_lo:    .equ 00e0h
address_hi:    .equ 00e1h
curbank:    .equ 00e2h
banksleft:    .equ 00e3h
ppuaddr: .equ 00e4h

uaddress:       .equ 00e6h
uaddress_lo:    .equ 00e6h
uaddress_hi:    .equ 00e7h

bank_lo:    .equ 00e8h
bank_hi:    .equ 00e9h

numpages_lo .equ 00eah
numpages_hi .equ 00ebh


temp_crc:    .equ 00ech

;plugin header defines
p_ver: = 2	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Namco 108/109/118/119"
             .db " + 74xx32 carts (INES 88)"
             .db " + opt. 74xx161 (INES 154)"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 64 >> 4,128 >> 4
             .db 128 >> 3,0 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

;mirroring is either 1-screen (mapper ctrl) or hardwired H/V

             ldx #00h		;x = H mirroring
             
             lda #020h
             jsr wr_ppu		;set PPU to $2000

             lda #055h
             sta 2007h
             lda #0aah
             sta 2007h		;write values 55h and AAh

             lda #024h
             jsr wr_ppu
             lda 2007h		;set PPU to $2400

             lda 2007h
             cmp #055h
             bne got_mir

             lda 2007h
             cmp #0aah
             bne got_mir	;compare first bytes to see if they are the same

             lda #020h
             jsr wr_ppu		;reset PPU to $2000

             lda #0aah
             sta 2007h		;write AAh first this time
             lda #055h
             sta 2007h

             lda #024h
             jsr wr_ppu
             lda 2007h		;set PPU to $2400 again

             lda 2007h
             cmp #0aah
             bne got_mir	;compare again

             inx		;is V mirroring

;check PPU $2800 as well, if that continues to be the same, it is 1-screen

             lda #028h
             jsr wr_ppu
             lda 2007h		;set PPU to $2800

             lda 2007h
             cmp #0aah
             bne got_mir	;compare, if different, is hard-wired

             lda 2007h
             cmp #055h
             bne got_mir

             ldx #4		;is 1-screen mirroring

got_mir:     txa
             jsr send_byte   ;send byte

dump_prg:    lda #0
             sta numpages_lo
             lda #1
             sta numpages_hi
             ldy #8
             sty curbank
             jsr comp_pbank
             beq got_prg	;64K

             asl numpages_hi	;128K
             lda #16
             sta curbank

got_prg:     lda numpages_lo
             jsr send_byte
             lda numpages_hi
             jsr send_byte
             lda #01h
             jsr send_byte	;send PRG header

             lda curbank
             sta banksleft
             lda #0
             sta curbank

next_pbank:  lda #6
             sta 8000h
             lda curbank
             sta 8001h
             inc curbank

             lda #0
             sta address_lo
             lda #080h
             sta address_hi

             ldx #32		;8K banks
             ldy #0

send_pbank:  lda (address),y
             jsr send_byte
             iny
             bne send_pbank
             inc address_hi
             dex
             bne send_pbank
             dec banksleft
             bne next_pbank

;determine CHR ROM size

got_chr:     lda #0		;send CHR header
             jsr send_byte
             lda #2		;128K CHR
             jsr send_byte
             lda #002h
             jsr send_byte

             lda #0
             sta curbank
             lda #64
             sta banksleft
            
next_chr:    lda curbank
             jsr wr_cbank
             inc curbank
             lda ppuaddr
             jsr wr_ppu
             lda 2007h

             ldx #8		;2K banks
             ldy #0

dump_chr:    lda 2007h
             jsr send_byte
             iny
             bne dump_chr
             dex
             bne dump_chr
             dec banksleft
             bne next_chr

done:        lda #0
             jsr send_byte
             lda #0
             jsr send_byte
             lda #0
             jsr send_byte

             rts

wr_cbank:    asl a
             sta bank_lo
             cmp #64
             bmi lower

upper:       lda #2
             sta 08000h
             lda bank_lo
             sta 08001h		;set first 1K

             lda #3
             sta 08000h
             lda bank_lo
             ora #1
             sta 08001h		;set second 1K

             lda #010h		;2K will be at PPU $1000
             sta ppuaddr
             rts

lower:       lda #0
             sta 08000h
             lda bank_lo
             sta 08001h		;set 2K

             lda #000h		;2K will be at PPU $0000
             sta ppuaddr
             rts


;y = bank to compare
;z=same
comp_pbank:  ldx #0
             stx bank_lo	;lower bank #
             sty bank_hi	;upper bank #
             sty banksleft	;# banks to compare

cpb_loop:    lda #000h		;init pointers to $8000 and $a000
             sta address_lo
             sta uaddress_lo
             lda #080h
             sta address_hi
             lda #0a0h
             sta uaddress_hi

             lda #007h		;set $a000 to PRG bank hi
             sta 08000h
             lda bank_hi
             sta 08001h

             lda #006h		;set $8000 to PRG bank lo
             sta 08000h
             lda bank_lo
             sta 08001h

             ldx #32		;8K banks
             ldy #0

cpb_loop2:   lda (address),y
             cmp (uaddress),y
             bne cpb_diff
             iny
             bne cpb_loop2
             inc address_hi
             inc uaddress_hi
             dex
             bne cpb_loop2
             inc bank_lo
             inc bank_hi
             dec banksleft
             bne cpb_loop

cpb_diff:    rts

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
